@@ -35,6 +35,16 @@ qb.mcmc <- function(cross,
   algorithm <- "M-H"
 
   cross.name <- deparse(substitute(cross))
+
+  ## Save cross object if it is a transient object.
+  tmp <- make.names(cross.name)
+  is.transient.cross <- tmp != cross.name
+  if(is.transient.cross) {
+    ## Should make sure the names is unique.
+    tmp <- make.dir.name(tmp, "")
+    assign(tmp, cross, pos = 1)
+    cross.name <- tmp
+  }
   
   if(is.null(cross$geno[[1]]$prob)) {
     warning("First running qb.genoprob with default step size",
@@ -45,7 +55,15 @@ qb.mcmc <- function(cross,
   n.ind = nind(cross)                   # number of individuals
   n.chr = nchr(cross)                   # number of chromsomes
   n.gen = dim(cross$geno[[1]]$prob)[3]  # number of genotypes
-
+##############################
+## For Multiple traits -- SB
+  multiple.trait = 0        #logical if TRUE multiple trait analysis is done
+  if(data$multiple.trait){ 
+  multiple.trait=1
+  if(tolower(data$multiple.type)=="traditional") multiple.trait=1 #redundant but being careful
+  if(tolower(data$multiple.type)=="sur") multiple.trait=2 
+   } 
+##########################
   ## loci on the genome
   loci <- pull.loci(cross)
   n.loci <- sapply(loci, length) # number of loci on each chr
@@ -61,6 +79,7 @@ qb.mcmc <- function(cross,
 
   ## data 
   pheno = data$pheno.col
+  n.pheno=length(pheno) ## For Multiple traits -- SB
   yvalue = data$yvalue
   if( data$trait == "normal" ) traittype = 1
   if( data$trait == "binary" ) traittype = 2
@@ -96,9 +115,22 @@ qb.mcmc <- function(cross,
   qtl_envi = model$qtl_envi                                   
   intcov = model$intcov
   depen = model$depen
-  prop = model$prop
+  prop = model$prop                                       
   contrast = model$contrast
-  
+##############################
+## For Multiple traits -- SB
+
+  if(multiple.trait==2) diff.loc=model$diff.loc else diff.loc=FALSE
+
+if(multiple.trait)
+{ ## Initial Value for variance-cov matrix (Done here since calculation of inverse
+  ##  is required
+   Y <- yvalue
+   Y[Y==999]=NA
+  sigma.initial <- var(Y,na.rm=TRUE,use="complete.obs")
+  sigma.initial <- solve(sigma.initial)
+}  
+##########################
 
   totaliter = n.iter*n.thin + n.burnin
 
@@ -111,7 +143,12 @@ qb.mcmc <- function(cross,
   }
 
   allTraits = colnames(cross$pheno)
-  output = output.dir( qbDir = mydir, traitName = allTraits[pheno] )	# set up a directory to save outputs
+  
+########### Single trait Code #################################################
+
+if(!multiple.trait)
+{
+output = output.dir( qbDir = mydir, traitName = allTraits[pheno] )	# set up a directory to save outputs
 
   z<-.C("RSingleTraitMCMCSetup",
 
@@ -185,6 +222,88 @@ qb.mcmc <- function(cross,
     data,
     model ) 
 
+
+  
+}
+
+########### Multiple trait Code #################################################
+
+if(multiple.trait)
+{
+   yvalue = as.matrix(yvalue)
+   dimnames(yvalue) <- NULL
+   qtlloc=1
+  if( algorithm == "M-H" ) algorithm = 0  else algorithm = 1
+
+output = output.dir( qbDir = mydir, traitName = allTraits[pheno][1] )	# set up a directory to save outputs
+
+  z<-.C("RMultipleTraitsMCMCSetup",
+
+  as.integer(n.ind),           # number of individuals
+  as.integer(n.chr),           # number of chromosomes
+  as.integer(n.gen),           # number of genotypes
+  as.integer(n.pheno),         # number of phenotypes to analyzed jointly
+  as.integer(n.loci),          # numbers of loci on chromosomes
+  as.double(loci),             # vector of loci
+  as.double(prob),             # vector of probabilities
+
+	as.vector(yvalue,mode="double"),                        # phenotypic value
+  as.integer(multiple.trait),   # logical if =1 mutiple trait analysis performed
+	as.integer(traittype),       # type of traits (1: normal, 2: binary, 3: ordinal)
+  as.integer(ncategory),       # number of categories for binary and ordinal traits    	
+  as.integer(n.iter),          # number of iterations
+	as.integer(n.thin),          # number of thin
+  as.integer(n.burnin),        # number of burnin
+  as.integer(algorithm),       # MCMC method (1: GIBBS, 0: MH sampler)
+	as.integer(genoupdate),      # 1: update QTL genotypes; 0: don't update QTL genotypes
+  as.integer(epis),            # epistatic model(epis=1) or nonepistatic model (epis=0)
+	as.integer(emainqtl),        # prior number of main-effect QTL
+	as.integer(eqtl),            # prior number of all QTL
+	as.integer(mnqtl),           # upper bound of detectable QTL
+	as.double(interval),         # distance of flanking QTL
+	as.integer(chrnqtl),         # maximum number of QTL on each chromosome
+  as.integer(envi),            # envi=1: include covariates, envi=0: covariates
+  as.integer(qtl_envi),        # qtl_envi=1: include g-by-e, qtl_envi=0: exclude g-by-e 
+	as.integer(nrancov),         # number of random covariates
+	as.integer(nfixcov),         # number of fixed covariates
+	as.integer(intcov),          # indicating which fixed covariates are included to interact QTL 
+  as.double(rancoef),          # random covariates
+  as.double(fixcoef),          # fixed covariates
+  as.integer(nran),            # numbers of levels for random covariates 
+	as.integer(depen),           # depen=1: use dependent prior for epistatic effects
+	as.double(prop),             # prior prob for three types of epistatic effects
+  as.integer(seed),	     # Seed specification for the pseudo-random number generator srand
+  as.integer(verbose),
+  as.integer(diff.loc),
+  as.integer(qtlloc),
+  PACKAGE="qtlbim")
+
+  if(verbose) {
+    cat("MCMC sample has been saved to: ")
+    cat(output)
+    cat(".\n")
+  }
+
+  ## create an object qb
+  qb = list(cross.name = cross.name,
+    cross = class(cross)[1],
+    output.dir = output, 
+    n.iter = n.iter,
+    n.pheno=n.pheno,
+    n.thin = n.thin,
+    n.burnin = n.burnin,
+    algorithm = algorithm, 
+    genoupdate = genoupdate,
+    step = attr(cross$geno[[1]]$prob, "step"),
+    seed = seed,
+    verbose = verbose)
+
+    qb=c(qb,data,model) 
+
+#   for(ph in 1:n.pheno) qb = qb.reorder( qb, pheno = ph )   # transfer mcmc output format for graphics
+
+} 
+
   ## Assign qb.genoprob attributes to args list if not there.
   defaults <- qb.genoprob.defaults(cross)
   for(i in names(defaults))
@@ -197,16 +316,20 @@ qb.mcmc <- function(cross,
     cat("Bayesian MCMC took ")
     cat(as.character(round(walltime, digits=2)))
     cat(" minutes. \n")
-  }
+    qb=c(qb,time=walltime) ## SB add for simulations
+    }
 
   qb = qb.reorder( qb )   # transfer mcmc output format for graphics
-
   gc()
 
   class(qb) = "qb"
 
   ## Reorganize as new qb object. For now do this at the end. Later do as created.
   qb <- qb.legacy(qb, remove = TRUE)
+
+  ## Remove temporary cross if it was from a transient object.
+  if(is.transient.cross)
+    remove(list = cross.name, pos = 1)
   
   qb
 }
@@ -264,12 +387,22 @@ qb.model <- function( cross, epistasis = TRUE,
 
   model = list( epistasis=epistasis, main.nqtl=main.nqtl, mean.nqtl=mean.nqtl, max.nqtl=max.nqtl,                                                 
                 interval=interval, chr.nqtl=chr.nqtl, qtl_envi=qtl_envi, intcov=intcov, 
-                depen=depen, prop=prop, contrast=contrast )
+                depen=depen, prop=prop, contrast=contrast,diff.loc=TRUE )
 
   gc()
   model
 }
 
+#######################################################################################################
+make.dir.name <- function(traitName, dtsep = "-")
+{
+  fullDt <- date()
+  dtMinusDay <- substr(fullDt, 5, nchar(fullDt))
+  dt <- gsub(":", "", dtMinusDay)
+  dt <- gsub(" ", dtsep, dt)
+  dt <- substr(dt, 1, nchar(dt) - 5)
+  paste(traitName, dt, sep = "_")
+}
 #######################################################################################################
 
 output.dir <- function( qbDir = getwd(), traitName = "trait1" )
@@ -279,14 +412,7 @@ output.dir <- function( qbDir = getwd(), traitName = "trait1" )
       stop("This output directory does not exist.", call. = FALSE)
    }
 
-#   dt = substring(date(),5,19)
-#   dt = gsub(" ","-",dt)
-   fullDt = date()
-   dtMinusDay = substr(fullDt,5,nchar(fullDt))
-   dt = gsub(":","",dtMinusDay)
-   dt = gsub(" ","-",dt)
-   dt = substr(dt,1,nchar(dt)-5)
-   traitName = paste(traitName,dt,sep="_")
+   traitName <- make.dir.name(traitName)
 
    traitDir<-file.path(qbDir,traitName)
   
@@ -300,6 +426,7 @@ output.dir <- function( qbDir = getwd(), traitName = "trait1" )
    pairFile = file.path(traitDir,"pairloci.dat")
    gbyeFile = file.path(traitDir,"gbye.dat")
    devFile = file.path(traitDir,"deviance.dat")
+   sigmaFile =file.path(traitDir,"sigma.dat") 
      
    z <- .C("ROutputManager",
            as.character(iterFile),
@@ -308,10 +435,9 @@ output.dir <- function( qbDir = getwd(), traitName = "trait1" )
            as.character(pairFile),
            as.character(gbyeFile),
            as.character(devFile),
+           as.character(sigmaFile),
            PACKAGE="qtlbim")
   
    return(traitDir) 
 }
 
-###########################################################################################################
-
